<?php

namespace Drupal\cascading_deletion;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;

/**
 * Class CascadingDeletionConfig.
 */
class CascadingDeletionManager implements CascadingDeletionManagerInterface {

  /**
   * @var array
   */
  private $config;

  /**
   * CascadingDeletionManager constructor.
   */
  public function __construct() {

    $this->setConfiguration();
  }

  /**
   * Set configuration from cache or new build.
   */
  private function setConfiguration() {

    // Check if data are in cache.
    $cid = 'cascading_deletion:' . \Drupal::languageManager()
        ->getCurrentLanguage()
        ->getId();
    $cache = \Drupal::cache()->get($cid);

    if ($cache) {
      $this->config = $cache->data;
    }
    else {
      $this->config = $this->buildConfiguration();

      // Cache the collected endpoints.
      \Drupal::cache()->set($cid, $this->config);
    }
  }

  /**
   * @return array
   */
  private function buildConfiguration() {

    // This array in used to manage deletion.
    $config = [];

    // Get all entity reference fields.
    $fieldsArray = \Drupal::service('entity_field.manager')
      ->getFieldMapByFieldType('entity_reference');

    // Loop on entity types.
    foreach ($fieldsArray as $entityTypeId => $fieldArray) {

      // Loop on fields.
      foreach ($fieldArray as $fieldName => $fieldSettings) {

        // Loop on bundles.
        foreach ($fieldSettings['bundles'] as $bundleName => $bundle) {

          // If is set cascading deletion, put it into array.
          $fieldConfigName = $entityTypeId . '.' . $bundleName . '.' . $fieldName;
          if (!is_null($fieldConfig = FieldConfig::load($fieldConfigName))) {
            $enabled = $fieldConfig->getThirdPartySetting('cascading_deletion', 'enabled');
            if ($enabled) {

              // Load Storage Config to get entity type target id.
              $fieldStorage = $fieldConfig->getFieldStorageDefinition();
              $parentEntityTypeId = $fieldStorage->getSetting('target_type');

              $config[$parentEntityTypeId][] = [
                'entity_type' => $entityTypeId,
                'field_name' => $fieldConfig->getName(),
              ];
            }
          }
        }
      }
    }

    return $config;
  }

  /**
   * @param $entityTypeId
   * @param $entityId
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function delete($entityTypeId, $entityId) {

    if (isset($this->config[$entityTypeId])) {

      foreach ($this->config[$entityTypeId] as $entityConfig) {

        $query = \Drupal::entityQuery($entityConfig['entity_type'])
          ->condition($entityConfig['field_name'], $entityId);

        $ids = $query->execute();

        // Delete all entities related to parent entity that will be deleted.
        if ($ids) {

          $storage_handler = \Drupal::entityTypeManager()->getStorage($entityConfig['entity_type']);
          $entities = $storage_handler->loadMultiple($ids);
          $storage_handler->delete($entities);
        }
      }
    }
  }
}
